/*
** guignome.h - header for Gnome GUI for the Elvis editor.
**
** Written by C W Rose.
**
** $Log$
*/

#ifndef lint
/* If this string is static GCC optimises it out of existence */
char id_guignome_h[] = "$Id: guignome.h,v 1.93 2008/01/01 12:00:00 cwr Exp $";
#endif

#ifndef EXTERN
#define EXTERN extern
#endif

/* This is sometimes easier than including all the main Elvis headers */
#ifndef MAP_ALL
#define MAP_ALL 0x011f
typedef unsigned int MAPFLAGS;
#endif

/* These are needed to build the standard Gnome program configuration,
** ie. gnome_program_init(), but it's difficult to fit them into the
** standard Elvis Makefile.
*/
#define PREFIX "/usr"
#define DATADIR "/usr/share"
#define SYSCONFDIR "/usr/etc"
#define LIBDIR "/usr/lib"

/* Standard defines */
#undef FALSE
#undef TRUE
#define FALSE 0
#define TRUE (!FALSE)

#define SAME 0

#define FAILED (-1)
#define NK 0
#define OK 1

#define S_BUF_SIZ 64
#define S_BUF_MAX (S_BUF_SIZ-1)
#define M_BUF_SIZ 128
#define M_BUF_MAX (M_BUF_SIZ-1)
#define BUF_SIZ 512
#define BUF_MAX (BUF_SIZ-1)

/* Local defines and their usage */

/* Font name buffer size */
#define F_SIZ 64
#define F_MAX (F_SIZ-1)

/* Filename linked list type */
#define DOCUMENTS 1
#define NOTEPADS 2

/* Documents list count */
#define DOC_COUNT 9
#define DOC_COUNT_S "9"
#define DOC_MAX 15
#define DOC_MAX_S "15"

/* Notepad list count; must match Documents list */
#define NOTE_COUNT DOC_COUNT

/* History command count */
#define HIST_COUNT 9

/* File menu filename entries */
#define ENTRY_COUNT 9

/* add_edwin()
** add_node()
** find_node()
#define FAILED (-1)
*/
#define FOUND (-2)
#define BEFORE (-3)
#define AFTER (-4)

#define E_NODE 2
#define R_NODE 3

/* filename_handler()
** step_ring()
*/
#define HERE 2
#define PREV 3
#define NEXT 4

/* clipread()
** mouse_handler()
*/
#define EMPTY 0
#define PARTIAL 1
#define FULL 2
#define FINAL 3

/* mouse_handler()
** toggle_cursor()
*/
#define NO 0
#define MAYBE 1
#define YES 2

/* mouse_handler() */
#define BEGIN 0
#define CURRENT 1
#define END 2

/* clipread()
** drop_data_handler()
** drag_drop_handler()
*/
#define NONE 0
#define GET_URI 1
#define GET_TXT 2
#define PUT_TXT 3

/* entry_handler()
** move_action()
*/
#define F_CMD 0
#define R_CMD 1
#define G_CMD 2

#define NO_MOVE 0
#define MOVE_ERR 1
#define FIND_NEXT 2
#define FIND_PREV 3
#define FIND_AGAIN 4
#define REPLACE 5
#define REPLACE_AGAIN 6
#define GOTO 7
#define GOTO_AGAIN 8
#define FILE_NAME 9	/* Re-use entry box for file name */

/* config_menus() */
#define MENU_UNDO 1
#define MENU_REDO 2
#define MENU_NEXT 4
#define MENU_PREV 8
#define MENU_REPL 16
#define MENU_GOTO 32
#define MENU_COPY 64
#define MENU_CUT 128
#define MENU_ENTRIES 8
#define MENU_MARK 256

/* print() functions */
#define P_HEAD_INDEX 0
#define P_BODY_INDEX 1
#define P_FLG_INDEX 2

#define HEADER_LINES 3.0
#define HEADER_MARGIN 5.0
#define FOOTER_LINES 2.0

#define SEPCHR '/'	/* Filesystem path separator */
#define SEPSTR "/"	/* Annoying */

/* Font class */
#define NO_FONT 0
#define TERMINAL_FONT 1
#define NORMAL_FONT 2
#define BOLD_FONT 3
#define ITALIC_FONT 4
#define PRINTER_FONT 5
#define HEADER_FONT 6
#define DEBUG_FONT 7

/* Color class */
#define BACK 0
#define FORE 1
#define NO_COLOR 0
#define FORE_COLOR 1
#define BACK_COLOR 2
#define CURSOR_COLOR 3

/* Default sizes */
#undef DEF_ROWS
#undef DEF_COLS
#define DEF_ROWS 24		/* VTE has initial rows and cols hard-wired */
#define DEF_ROWS_S "24"
#define DEF_COLS 80		/* so don't change the defaults, regardless */
#define DEF_COLS_S "80"
#define MIN_ROWS 2		/* set in Elvis' eventresize() */
#define MIN_ROWS_S "2"
#define MIN_COLS 30		/* set in Elvis' eventresize() */
#define MIN_COLS_S "30"
#define MAX_ROWS 500
#define MAX_ROWS_S "500"
#define MAX_COLS 2000
#define MAX_COLS_S "2000"

/* Structures and typedefs */

/* Gnome GUI interface */
struct GnomeGui {
	struct GnomeGui *next;
	struct GnomeGui *prev;
	GtkWidget *widget;	/* Spare */
	GtkWidget *toplevel;
	GtkWidget *gwindow;
	GtkWidget *statusbar;
	GtkWidget *msgbar;
	GtkWidget *msgbarbox;
	GtkWidget *entry;
	GtkWidget *entrybox;
	GtkWidget *hpaned;
	GtkWidget *barbox;
	GtkWidget *indicators;
	GtkWidget *holder;
	GtkWidget *scrollwin;
	GtkWidget *menubar;
	GtkWidget *notesmenu;
	GtkWidget *mousemenu;
	GtkWidget *toolbar;
	GtkTooltips *tooltips;
	GtkTooltips *menutips;
	GtkAccelGroup *acceltab;
	BonoboDockItem *tooldock;
	GtkActionGroup *mainactiongroup;
	GtkActionGroup *mouseactiongroup;
	GtkUIManager *mainuimanager;
	GtkUIManager *mouseuimanager;
	GConfClient *gconf_client;
	GnomeClient *gnome_client;
	GnomeProgram *gnome_program;
	GdkWindowState window_state;

	uid_t uid;		/* Process' actual uid */
	guint gconf_id;		/* Gconf connection id */
	guint glog_id;		/* Glib log handler id */
	guint snoop_id;		/* Key snooper handler id */
	guint mouse_ui_id;	/* Mouse menu merge id */
	guint entry_id;		/* Entry box activate signal id */
	guint edwin_id;		/* ID of current EDWIN node */

	int cfgdir_flg;		/* Config directory available */
	int scheme_flg;		/* Default color scheme set */
	int block_sel;		/* Text block selected */
	int caps_set;		/* Capslock key state */

	int cur_cmd;		/* Elvis' current command (enum EXCMD) */
	int move_cmd;		/* Current move action command */
	gchar history_name[F_SIZ];/* Move command history name (id) */

	GIOChannel *gio_out;	/* GIOChannel for shell commands' output */

	int drop_type;		/* Drop can be URI or text */
	guchar *drop_text;	/* Pointer to dropped text */
	guint drop_len;		/* Length of dropped text */
	int drop_fd;		/* fd of dropped file */

	int main_start_x;	/* Main window starting position */
	int main_start_y;
	int main_height;	/* Main window height and width */
	int main_width;
	int stepper_height;	/* Scrollbar stepper height and width */
	int stepper_width;

	GdkDisplay *display;	/* Toplevel display */
	GdkScreen *screen;	/* Toplevel screen */
	GdkCursor *cursor;	/* Toplevel cursor */
	GtkStyle *style;	/* Toplevel style */
	GtkIconTheme *theme;	/* Icon theme */
	char theme_name[F_SIZ];	/* Icon theme name */
	PangoContext *pango_context;	/* Toplevel pango_context */
	PangoLanguage *pango_lang;	/* Toplevel pango language */
	char def_fontname[F_SIZ];	/* Default terminal font name */
	char cur_fontname[F_SIZ];	/* Current terminal font name */
	char def_print_font[F_SIZ];	/* Default body printing font name */
	char cur_print_font[F_SIZ];	/* Current body printing font name */
	char def_hdr_print_font[F_SIZ]; /* Default header printing font name */
	char cur_hdr_print_font[F_SIZ]; /* Current header printing font name */
	char def_dbg_fontname[F_SIZ];	/* Default debug font name */
	char cur_dbg_fontname[F_SIZ];	/* Current debug font name */
	PangoFontDescription *pango_desc;
	PangoFontMetrics *pango_metrics;
	int pango_width;
	int pango_height;

	int use_def_font;	/* Dialog default button settings */
	int use_def_color;
	int use_def_cursor;
	int use_def_print;

	int font_codes[32];	/* Elvis' font codes */
	int color_types[32];	/* Elvis' color type */
	GdkColor vte_colors[32];/* VTE terminal colors */

	GdkColor def_forecolor;	/* Default colors */
	GdkColor def_backcolor;
	GdkColor def_cursorcolor;
	char def_forename[F_SIZ];/* Default color names */
	char def_backname[F_SIZ];
	char def_cursorname[F_SIZ];
	int def_cursorblink;	/* Default cursor blink setting */
	int def_hdrprint;	/* Default header print setting */

	GdkColor cur_forecolor;	/* Current colors */
	GdkColor cur_backcolor;
	GdkColor cur_cursorcolor;
	char cur_forename[F_SIZ];/* Current color names */
	char cur_backname[F_SIZ];
	char cur_cursorname[F_SIZ];
	int cur_cursorblink;	/* Current cursor blink setting */
	int cur_hdrprint;	/* Current header print setting */

	GtkWidget *horiz_bar;	/* Scrollbars and settings */
	GtkWidget *vert_bar;
	GtkRange *horiz_range;
	GtkRange *vert_range;
	GtkAdjustment *horiz_adj;
	GtkAdjustment *vert_adj;

	GtkWidget *print_dialog;/* Printer and page settings */
	GtkPrintSettings *print_settings;
	GtkPrintOperation *print_operation;

	char *prog_name;	/* Program name (argv[0]) */
	char lastprint[PATH_MAX+1]; /* Name of last file printed (full path) */
	char dictionary[F_SIZ];	/* Dictionary program name (no path) */
	char spellcheck[F_SIZ];	/* Spellcheck program name (no path) */
	GtkWidget *vterm;	/* Placeholder for initialisation */
	GdkGeometry def_hints;	/* Placeholder size adjustment hints */

	int def_rows;		/* Default terminal size */
	int def_cols;
	int bar_cols;		/* Columns used by scrollbar */

	unsigned int menu_state;/* Main menu item sensitivities */

	struct Edwin *edhead;	/* Start of linked list of edit windows */
	int edcount;		/* Number of windows on the list */

	struct Enode *docshead;	/* List of current documents */
	int docscount;		/* Number of documents on the list */

	struct Enode *noteshead;/* List of current notepads */
	int notescount;		/* Number of notepads on the list */
  };
typedef struct GnomeGui GNGUI;

/* File editing interface */
struct Edwin {
	unsigned int id;	/* Type of node */
	struct Edwin *prev;	/* Next in file edit ring */
	struct Edwin *next;	/* Previous in file edit ring */
	GNGUI *gn_gui;		/* Gnome GUI for this file (shared) */
	struct Enode *argshead;	/* List of current files */
	int argscount;		/* Number of files on the list */
	guint st_id;		/* This status window's ID */
	guint ms_id;		/* This message window's ID */
	char *edname;		/* This file's (full) name */
	char *edtitle;		/* This file's (short) title */
	char ro_flg;		/* File read-only flag */

	char keybuff[M_BUF_SIZ];/* Buffer for keystrokes */
	int keycount;		/* Number of characters in key buffer */

	GtkWidget *vterm;	/* This file's Gnome terminal widget */
	GtkWidget *eventbox;	/* Terminal widget's parking space */
	guint key_id;		/* This window's key handler signal id */
	guint mouse_dn_id;	/* This window's mouse button down signal id */
	guint mouse_up_id;	/* This window's mouse button up signal id */
	guint mouse_mv_id;	/* This window's mouse movement signal id */
	guint scroll_id;	/* This window's (mouse) scroll signal id */

	unsigned int attr_mask;	/* Attribute bit mask last used */
	unsigned int clr_mask;	/* Colour bitmask last used */
				/* Elvis uses signed values for colors */
	long int fg_clr;	/* Foreground color last used by Elvis */
	long int bg_clr;	/* Background color last used by Elvis */

	int use_def_font;	/* Dialog default button settings */
	int use_def_color;
	int use_def_cursor;

	GdkColormap *colormap;	/* This window's colormap */
	GdkColor forecolor;	/* This window's foreground color */
	GdkColor backcolor;	/* This window's background color */
	GdkColor cursorcolor;	/* This window's cursor color */
	int cursorblink;	/* This windows's cursor blink setting */

	char fontname[F_SIZ];	/* Pango font name */
	PangoLanguage *pango_lang; /* Related font info */
	PangoContext *pango_context;
	PangoFontDescription *pango_desc;
	PangoFontMetrics *pango_metrics;
	int pango_width;
	int pango_height;

	GdkGeometry hints;	/* Terminal cell size adjustments */
	int xpad;
	int ypad;
	glong cols;		/* Terminal size */
	glong rows;
  };
typedef struct Edwin EDWIN;

/* Linked list node template */
struct Node {
	unsigned int id;	/* Type of node */
	struct Node *next;	/* Next node */
	struct Node *prev;	/* Previous node */
	void *data;		/* Data */
	char *name;		/* Name */
	char *label;		/* Label */
  };
typedef struct Node NODE;

/* File editing window node */
struct Enode {
	unsigned int id;	/* Type of node */
	struct Enode *next;	/* Next file name */
	struct Enode *prev;	/* Previous file name */
	void *data;		/* NULL */
	char *name;		/* Long filename */
	char *label;		/* Short filename */
	guint signal;		/* Signal used by GtkAction */
	guint uiid;		/* Merge id used by UIManager */
  };
typedef struct Enode ENODE;

/* File printing interface */
struct Printjob {
	PangoLanguage *pango_lang;	/* Pango data for header and body */
	PangoContext *pango_context;
	PangoFontDescription *pango_head_desc;
	PangoFontDescription *pango_body_desc;
	PangoFontMetrics *pango_head_metrics;
	PangoFontMetrics *pango_body_metrics;
	double head_char_height;	/* Font data for header and body */
	double body_char_height;
	gint head_font_size;
	gint body_font_size;
	FILE *fpin;			/* Input and temp file pointer */
	char filename[PATH_MAX+1];	/* Name of file to be printed */
	char *tmpname;			/* Name of temporary file */
	char upper_header[PATH_MAX+1];	/* First line of header */
	char lower_header[M_BUF_SIZ];	/* Second line of header */
	char *buff;			/* Input line buffer */
	gint line_size;			/* Length of longest line */
	gint line_count;		/* Number of lines in file */
	gint page_count;		/* Number of pages in file */
	gint lines_per_page;		/* Lines per page */
	gint current_line;		/* Current file line number */
  };
typedef struct Printjob PRINTJOB;

/* Key translation table entry */
struct Keyvalue {
	guint	keyval;		/* Symbols are from gdk/gdkkeysyms.h */
	int	ctrlstate;	/* -1 = ignore, 0 = must be 0, 1 = must be 1 */
	int	altstate;	/* -1 = ignore, 0 = must be 0, 1 = must be 1 */
	int	nchars;		/* Number of valid chars, or 0 for table end */
	char	* keystr;	/* Keystring output */
  };
typedef struct Keyvalue KEY;

/* Default key mapping table entry.
**
** From X11/X.h: typedef unsigned long XID; typedef XID KeySym;
** KeySym symbols are from gdk/gdkkeysyms.h.
** From elvis/map.h: typedef unsigned int MAPFLAGS;
*/
struct Mapkey {
	char *label;
	KeySym sym;
	char *cooked;
	MAPFLAGS flags;
  };
typedef struct Mapkey MAPKEY;

/* Global variables, initialised in guignome.c */
EXTERN GNGUI gn_gui;		/* Gnome's version of the GUI */

/* Function prototypes */

/*
** EOF
*/

